# Managed API for Service Now
Managed API for Service Now is an API Client for Service Now by [Adaptavist](https://www.adaptavist.com/) primarily designed for the use in [ScriptRunner Connect](https://scriptrunnerconnect.com). You can read more about Managed APIs [here](https://docs.adaptavist.com/src/managed-apis).

This is a runtime agnostic version of the Managed API (with `core` suffix) that makes it easy to port it to any runtime you may want to use this Managed API on.

# Platform specific ports
- [ScriptRunner Connect](https://www.npmjs.com/package/@managed-api/service-now-sr-connect)

# Supported API calls
- [fetch](https://docs.adaptavist.com/src/managed-apis/managed-api-abstractions)
- Attachment
    - [deleteAttachment](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-DELETE)
    - [getAttachments](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET)
    - [getMetadataForAttachment](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET-file)
    - [uploadBinaryFile](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-file)
    - [uploadMultipartFile](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-upload)
- Table
    - [addRecord](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-POST)
    - [deleteRecord](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-DELETE)
    - [getRecord](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET-id)
    - [getRecords](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET)
    - [updateRecord](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PUT)
    - [updateRecordWithNameValuePair](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PATCH)

# Porting to other runtimes
While the primary reason for creating Managed APIs was to use them in [ScriptRunner Connect](https://scriptrunnerconnect.com) runtime, then you can easily port them to any runtime of your choosing.

You can do so by extending the `*Core` Managed API class where you'll be asked to provide implementation for `getPlatformImplementation` function. In this implementation you have yo specify how to perform buffer encoding/decoding operations and how to perform HTTP calls:
```typescript
{
    buffer: {
        encode(input: string): ArrayBuffer;
        decode(input: ArrayBuffer): string;
    },
    performHttpCall(request: Request): Promise<Response>
}
```

Here is an example how to create a [Node](https://nodejs.org/en/) port of the Managed API:
```typescript
import { ServiceNowApiCore } from "@managed-api/service-now-core";
import { PlatformImplementation, Request, Response, Headers } from "@managed-api/commons-core";
import fetch from "node-fetch";

export class ServiceNowApi extends ServiceNowApiCore {
    constructor(private baseUrl: string, private authToken: string) {
        super();
    }

    protected getPlatformImplementation(): PlatformImplementation {
        return {
            buffer: {
                encode: (input) => Buffer.from(input, 'utf-8'),
                decode: (input) => Buffer.from(input).toString('utf-8')
            },
            performHttpCall: (request) => this.performHttpCall(request)
        };
    }

    private async performHttpCall(request: Request): Promise<Response> {
        const requestUrl = `${this.baseUrl}${request.url}`; // Substitute base URL
        request.headers.set('Authorization', `Basic ${this.authToken}`); // Substitute auth token
        const response = await fetch(requestUrl, {
            method: request.method,
            headers: request.headers as any,
            body: request.method.toLowerCase() !== 'get' ? Buffer.from((await request.arrayBuffer()) ?? new ArrayBuffer(0)) : undefined
        });

        const apiHeaders = new Headers();
        response.headers.forEach((value, key) => apiHeaders.append(key, value));

        const body = await response.arrayBuffer();

        return super.buildResponse(response.url, response.status, response.statusText, apiHeaders, body);
    }
}
```

Please note that the authentication handling details may not be accurate and serve as an example. If you wish yo use these Managed APIs without having to concern yourself how to deal with and secure authentication details when working with APIs, then feel free to evaluate [ScriptRunner Connect](https://scriptrunnerconnect.com) platform.

# Contact
At this time Managed APIs are unlicensed and close sourced, but feel free to drop an email to [eknoll@adaptavist.com](mailto://eknoll@adaptavist.com) for ideas, suggestions and improvements.

# Changelog

* 2.1.0
    - Added `Content-Type: application/octet-stream` header to [Upload Binary File to a Record](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-file) that can be used as `Attachment.uploadBinaryFile` or `All.uploadBinaryFile`.
* 2.0.0
    - `Breaking change:` URL and query string parameters are now URL encoded, meaning if you encoded them explicitly before you have to remove your own encoding, otherwise they will get double encoded.

* 0.3.0
    - Added support for the following Attachment methods:
        * [Delete Attachment](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-DELETE) that can be used as `Attachment.deleteAttachment` or `All.deleteAttachment`.
        * [Get Attachments](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET) that can be used as `Attachment.getAttachments` or `All.getAttachmentst`.
        * [Get Attachment](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET-sys_id) that can be used as `Attachment.getMetadataForAttachment` or `All.getMetadataForAttachment`.
        * [Upload Binary File to a Record](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-file) that can be used as `Attachment.uploadBinaryFile` or `All.uploadBinaryFile`.
        * [Upload Multipart File Attachment](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-upload) that can be used as `Attachment.uploadMultipartFile` or `All.uploadMultipartFile`.
    - Added support for the following Table methods:
        * [Delete Record from Table](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-DELETE) that can be used as `Table.deleteRecord` or `All.deleteRecord`.
        * [Retrieve multiple Records for the Table](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET) that can be used as `Table.getRecords` or `All.getRecords`.
        * [Retrieve a Tecord](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET-id) that can be used as `Table.getRecord` or `All.getRecord`.
        * [Update a Tecord with the name-value pairs](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PATCH) that can be used as `Table.updateRecordWithNameValuePair` or `All.updateRecordWithNameValuePair`.
        * [Insert a Record in the Table](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-POST) that can be used as `Table.addRecord` or `All.addRecord`.
        * [Update a Record](https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PUT) that can be used as `Table.updateRecord` or `All.updateRecord`.

* 0.1.5
    - Fixed README

Copyright Adaptavist 2023 (c) All rights reserved