/*!
 * Copyright Adaptavist 2023 (c) All rights reserved
 */
import { Response, BaseApiCore, Request, constructHeaders, executeHttpCall, constructUrl, FormData, Blob } from '@managed-api/commons-core';
import { customErrorHandlers, errorStrategyBuilderConstructor } from './errorStrategy';
import { errorMessageBuilder } from './common';
;
export class ServiceNowApiCore extends BaseApiCore {
    constructor(options) {
        var _a, _b;
        super();
        this.options = options;
        if (!((_a = this.options) === null || _a === void 0 ? void 0 : _a.errorStrategy) && ((_b = this.options) === null || _b === void 0 ? void 0 : _b.errorStrategy) !== null) {
            this.setGlobalErrorStrategy((builder) => builder.retryOnRateLimiting(-1));
        }
        this.Attachment = new AttachmentGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.Table = new TableGroup(this.implementation, () => this.getGlobalErrorStrategy());
        this.All = new AllGroup(this.Attachment, this.Table);
    }
    setGlobalErrorStrategy(errorStrategy) {
        var _a;
        this.options = {
            ...(_a = this.options) !== null && _a !== void 0 ? _a : {},
            errorStrategy
        };
    }
    buildResponse(url, status, statusText, headers, body) {
        const response = new Response(url, status, statusText, headers, body);
        response.setBufferImplementation(this.implementation);
        return response;
    }
    getGlobalErrorStrategy() {
        var _a;
        return (_a = this.options) === null || _a === void 0 ? void 0 : _a.errorStrategy;
    }
}
class AllGroup {
    constructor(Attachment, Table) {
        this.Attachment = Attachment;
        this.Table = Table;
    }
    /**
     * This method deletes the attachment with a specific sys_id value.
     * Alternative usage: Attachment.deleteAttachment
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-DELETE
     * @param options Request options.
     */
    async deleteAttachment(options) {
        return this.Attachment.deleteAttachment(options);
    }
    /**
     * Returns the metadata for multiple attachments.
     * Alternative usage: Attachment.getAttachments
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET
     * @param options Request options.
     */
    async getAttachments(options) {
        return this.Attachment.getAttachments(options);
    }
    /**
     * Returns the metadata for the attachment file with a specific sys_id value.
     * Alternative usage: Attachment.getMetadataForAttachment
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET-sys_id
     * @param options Request options.
     */
    async getMetadataForAttachment(options) {
        return this.Attachment.getMetadataForAttachment(options);
    }
    /**
     * Uploads a specified binary file as an attachment to a specified record.
     * Alternative usage: Attachment.uploadBinaryFile
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-file
     * @param options Request options.
     */
    async uploadBinaryFile(options) {
        return this.Attachment.uploadBinaryFile(options);
    }
    /**
     * Deletes the specified record from the specified table.
     * Alternative usage: Table.deleteRecord
     * Documentation:https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-DELETE
     */
    async deleteRecord(options) {
        return this.Table.deleteRecord(options);
    }
    /**
     * Retrieves multiple records for the specified table.
     * Alternative usage: Table.getRecords
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET
     * @param options Request options.
     */
    async getRecords(options) {
        return this.Table.getRecords(options);
    }
    /**
     * Retrieves the record identified by the specified sys_id from the specified table.
     * Alternative usage: Table.getRecord
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET-id
     * @param options Request options.
     */
    async getRecord(options) {
        return this.Table.getRecord(options);
    }
    /**
     * Updates the specified record with the name-value pairs included in the request body.
     * Alternative usage: Table.updateRecordWithNameValuePair
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PATCH
     * @param options Request options.
     */
    async updateRecordWithNameValuePair(options) {
        return this.Table.updateRecordWithNameValuePair(options);
    }
    /**
     * Inserts one record in the specified table. Multiple record insertion is not supported by this method.
     * Alternative usage: Table.addRecord
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-POST
     * @param options Request options.
     */
    async addRecord(options) {
        return this.Table.addRecord(options);
    }
    /**
     * Updates the specified record with the request body.
     * Alternative usage: Table.updateRecord
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PUT
     * @param options Request options.
     */
    async updateRecord(options) {
        return this.Table.updateRecord(options);
    }
}
class AttachmentGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * This method deletes the attachment with a specific sys_id value.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-DELETE
     * @param options Request options.
     */
    async deleteAttachment(options) {
        const { sys_id, headers, errorStrategy } = options;
        const request = new Request('DELETE', constructUrl(`/api/now/v1/attachment/${encodeURIComponent(sys_id)}`), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Returns the metadata for multiple attachments.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET
     * @param options Request options.
     */
    async getAttachments(options) {
        const { sysparm_limit, sysparm_offset, sysparm_query, headers, errorStrategy } = options !== null && options !== void 0 ? options : {};
        const queryParams = { sysparm_limit, sysparm_offset, sysparm_query };
        const request = new Request('GET', constructUrl('api/now/v1/attachment', queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Returns the metadata for the attachment file with a specific sys_id value.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-GET-file
     *  @param options Request options.
     */
    async getMetadataForAttachment(options) {
        const { sys_id, headers, errorStrategy } = options;
        const request = new Request('GET', `/api/now/v1/attachment/${encodeURIComponent(sys_id)}`, constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Uploads a specified binary file as an attachment to a specified record.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-file
     * @param options Request options.
     */
    async uploadBinaryFile(options) {
        const { encryption_context, file_name, table_name, table_sys_id, body, headers, errorStrategy } = options;
        const queryParams = { encryption_context, file_name, table_name, table_sys_id };
        const request = new Request('POST', constructUrl('/api/now/v1/attachment/file', queryParams), constructHeaders(body, { 'Content-Type': 'application/octet-stream', ...headers, }), body);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Uploads a multipart file attachment.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_AttachmentAPI.html#title_attachment-POST-upload
     * @param options Request options.
     */
    async uploadMultipartFile(options) {
        const { body, headers, errorStrategy } = options;
        const formBody = new FormData();
        formBody.setBufferImplementation(this.implementation);
        formBody.append('table_name', body.table_name, '');
        formBody.append('table_sys_id', body.table_sys_id, '');
        formBody.append('uploadFile', new Blob(this.implementation, [body.uploadFile], {}), body.file_name);
        const bodyBlob = formBody.getBlob();
        const request = new Request('POST', constructUrl('/api/now/v1/attachment/upload'), constructHeaders(bodyBlob, headers), bodyBlob);
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
class TableGroup {
    constructor(implementation, getGlobalErrorStrategy) {
        this.implementation = implementation;
        this.getGlobalErrorStrategy = getGlobalErrorStrategy;
    }
    /**
     * Deletes the specified record from the specified table.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-DELETE
     * @param options Request options.
     */
    async deleteRecord(options) {
        const { sysparm_query_no_domain, tableName, sys_id, headers, errorStrategy } = options;
        const queryParams = { sysparm_query_no_domain };
        const request = new Request('DELETE', constructUrl(`/api/now/v1/table/${encodeURIComponent(tableName)}/${encodeURIComponent(sys_id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers, true);
    }
    /**
     * Retrieves multiple records for the specified table.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET
     * @param options Request options.
     */
    async getRecords(options) {
        const { tableName, headers, errorStrategy, ...queryParams } = options;
        const request = new Request('GET', constructUrl(`/api/now/v1/table/${encodeURIComponent(tableName)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Retrieves the record identified by the specified sys_id from the specified table.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-GET-id
     * @param options Request options.
     */
    async getRecord(options) {
        const { sysparm_query_no_domain, tableName, headers, errorStrategy, sysparm_display_value, sysparm_exclude_reference_link, sysparm_fields, sysparm_view, sys_id } = options;
        const queryParams = { sysparm_display_value, sysparm_exclude_reference_link, sysparm_fields, sysparm_query_no_domain, sysparm_view };
        const request = new Request('GET', constructUrl(`/api/now/v1/table/${encodeURIComponent(tableName)}/${encodeURIComponent(sys_id)}`, queryParams), constructHeaders(headers));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates the specified record with the name-value pairs included in the request body.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PATCH
     * @param options Request options.
     */
    async updateRecordWithNameValuePair(options) {
        const { sysparm_query_no_domain, tableName, headers, errorStrategy, sysparm_display_value, sysparm_input_display_value, sysparm_fields, sysparm_view, sys_id, body } = options;
        const queryParams = { sysparm_display_value, sysparm_fields, sysparm_query_no_domain, sysparm_view, sysparm_input_display_value };
        const request = new Request('PATCH', constructUrl(`/api/now/v1/table/${encodeURIComponent(tableName)}/${encodeURIComponent(sys_id)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Inserts one record in the specified table. Multiple record insertion is not supported by this method.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-POST
     * @param options Request options.
     */
    async addRecord(options) {
        const { tableName, headers, errorStrategy, body, sysparm_display_value, sysparm_exclude_reference_link, sysparm_fields, sysparm_input_display_value, sysparm_view } = options;
        const queryParams = { sysparm_display_value, sysparm_exclude_reference_link, sysparm_fields, sysparm_input_display_value, sysparm_view };
        const request = new Request('POST', constructUrl(`/api/now/v1/table/${encodeURIComponent(tableName)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
    /**
     * Updates the specified record with the request body.
     * Documentation: https://docs.servicenow.com/bundle/utah-api-reference/page/integrate/inbound-rest/concept/c_TableAPI.html#title_table-PUT
     * @param options Request options.
     */
    async updateRecord(options) {
        const { tableName, sys_id, headers, errorStrategy, body, sysparm_display_value, sysparm_exclude_reference_link, sysparm_fields, sysparm_input_display_value, sysparm_query_no_domain, sysparm_view } = options;
        const queryParams = { sysparm_display_value, sysparm_exclude_reference_link, sysparm_fields, sysparm_input_display_value, sysparm_view, sysparm_query_no_domain };
        const request = new Request('PUT', constructUrl(`/api/now/v1/table/${encodeURIComponent(tableName)}/${encodeURIComponent(sys_id)}`, queryParams), constructHeaders(body, headers), JSON.stringify(body));
        request.setBufferImplementation(this.implementation);
        return await executeHttpCall(this.implementation, request, errorStrategyBuilderConstructor, this.getGlobalErrorStrategy(), errorStrategy, errorMessageBuilder, customErrorHandlers);
    }
}
